@@ -7,24 +7,30 @@
 --
 -- The default 'ErrorHandler' and 'TerminationHandler' both throw a 'GetException'.
 
-module Data.Conduit.Cereal (GetException, sinkGet, conduitGet, sourcePut) where
+module Data.Conduit.Cereal ( GetException
+                           , sinkGet
+                           , conduitGet
+                           , sourcePut
+                           , conduitPut
+                           ) where
 
-import           Control.Monad.Exception
+import           Control.Exception.Base
 import qualified Data.ByteString as BS
 import qualified Data.ByteString.Lazy as LBS
 import qualified Data.Conduit as C
-import           Data.Conduit.Cereal.Internal
-import           Data.Conduit.List (sourceList)
+import qualified Data.Conduit.List as CL
 import           Data.Serialize hiding (get, put)
 import           Data.Typeable
 
+import           Data.Conduit.Cereal.Internal
+
 data GetException = GetException String
   deriving (Show, Typeable)
 
 instance Exception GetException
 
 -- | Run a 'Get' repeatedly on the input stream, producing an output stream of whatever the 'Get' outputs.
-conduitGet :: MonadException m => Get output -> C.Conduit BS.ByteString m output
+conduitGet :: C.MonadThrow m => Get output -> C.Conduit BS.ByteString m output
 conduitGet = mkConduitGet errorHandler 
   where errorHandler msg _ = pipeError $ GetException msg 
 
@@ -32,14 +38,18 @@ conduitGet = mkConduitGet errorHandler
 --
 -- If 'Get' succeed it will return the data read and unconsumed part of the input stream.
 -- If the 'Get' fails due to deserialization error or early termination of the input stream it raise an error.
-sinkGet :: MonadException m => Get r -> C.Sink BS.ByteString m r
+sinkGet :: C.MonadThrow m => Get r -> C.Sink BS.ByteString m r
 sinkGet = mkSinkGet errorHandler terminationHandler 
   where errorHandler msg _ = pipeError $ GetException msg 
         terminationHandler f _ = let Fail msg = f BS.empty in pipeError $ GetException msg 
 
-pipeError :: (MonadException m, Exception e) => e -> C.Pipe i o m r
-pipeError e = C.PipeM (throw e) undefined 
+pipeError :: (C.MonadThrow m, Exception e) => e -> C.Pipe i o m r
+pipeError e = C.PipeM (C.monadThrow e) undefined 
 
 -- | Convert a 'Put' into a 'Source'. Runs in constant memory.
 sourcePut :: Monad m => Put -> C.Source m BS.ByteString
-sourcePut put = sourceList $ LBS.toChunks $ runPutLazy put
+sourcePut put = CL.sourceList $ LBS.toChunks $ runPutLazy put
+
+-- | Run a 'Putter' repeatedly on the input stream, producing a concatenated 'ByteString' stream.
+conduitPut :: Monad m => Putter input -> C.Conduit input m BS.ByteString
+conduitPut p = CL.map $ runPut . p
